<?php
namespace Service;

use Zeedhi\Framework\DTO\Response\Message;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\ORM\DateTime;

class ErpService extends \Zeedhi\Framework\Controller\Simple {

    protected $connection;
    protected $entityManager;

    public function __construct(\Doctrine\ORM\EntityManager $entityManager)  {
        $this->connection = $entityManager->getConnection();
        $this->entityManager = $entityManager;
    }

    /**
     * consumirNotas
     */
    public function consumirNotas($row) {
        try {
            
            $arrayNotas = array();
            $notas = $this->getNotas($row);
            
            foreach($notas as $nota){
            
                $notaCompleta = $nota;
                $notaCompleta['ITEM'] = $this->getItens($nota);
                
                array_push($arrayNotas, $notaCompleta);
                
            }
            
            return $arrayNotas;
            
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function getNotas($row){
        try{
            $queryNotas = "SELECT DISTINCT(N.NRLANCTONF),
                                  N.IDBLOQUEADO,
                                  N.CDFILIAL,
                                  N.NRINSJURNF AS CNPJDEST,
                                  DECODE(N.IDORIGDESTNF, 'C' ,C.NMRAZSOCCLIE, 'F',F.NMRAZSOCFORN, FI.NMFILIAL) AS NMRAZSOCDEST,
                                  N.CDTIPOOPER,
                                  T.NMTIPOOPER,
                                  N.VRNOTAFISC,
                                  N.DTENTRSAID
                            FROM NOTAFISCAL N, CLIENTE C, FORNECEDOR F, FILIAL FI, TIPOOPERAC T
                            WHERE N.DTENTRSAID = :P_DTSAIDA
                            AND N.CDTIPOOPER = T.CDTIPOOPER
                            AND T.IDENTRSAIDOP = 'S'
                            AND N.CDCLIENTE       = C.CDCLIENTE(+)
                            AND N.CDFORNECED      = F.CDFORNECED(+)
                            AND N.CDFILIRECE      = FI.CDFILIAL(+)
                            AND N.IDSISTEMA IN ('EXP')
                            ORDER BY NRLANCTONF DESC";

			$params = array(
                'P_DTSAIDA' => $row['DTSAIDA']
            );
            $notas = $this->entityManager->getConnection()->fetchAll($queryNotas, $params);
            return $notas;
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function getItens($nota){
        try{
            $queryItens = "SELECT NRSEQITPROD,
                              CDPRODUTO,
                              DSPRODEMIS AS NMPRODUTO,
                              NRPEDIDO,
                              QTITEMPROD AS QTITEM,
                              VRUNITITEM,
                              VRDESCITEM,
                              VRTOTAITEM,
                              NRLOTEESTQ
                        FROM ITEMPRODUTO
                        WHERE CDFILIAL = :P_CDFILIAL
                        AND NRLANCTONF = :P_NRLANCTONF
                        ORDER BY NRSEQITPROD";
			$params = array(
                'P_CDFILIAL'     => $nota['CDFILIAL'],
                'P_NRLANCTONF'  => $nota['NRLANCTONF']
            );
            $itens = $this->entityManager->getConnection()->fetchAll($queryItens, $params);
            return $itens;
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function updateNotaEnviada($notas) {
        foreach($notas as $nota){
            
            $bloqueiaNota = "UPDATE NOTAFISCAL
                            SET IDBLOQUEADO = 'S'
                            WHERE CDFILIAL = :P_CDFILIAL
                            AND NRLANCTONF = :P_NRLANCTONF";

			$params = array(
                'P_CDFILIAL'    => $nota['CDFILIAL'],
                'P_NRLANCTONF'  => $nota['NRLANCTONF']
            );
            
            $resultQuery = $this->entityManager->getConnection()->executeQuery($bloqueiaNota, $params);
            
        }
    }
    
    /**
     * desbloqueiaCancNF
     */
    
    public function desbloqueiaCancNF($row) {
        try {
            
            $nota = $this->getNotaBloqueada($row);
            
            if(count($nota) > 0){
                $this->desbloqueiaNota($row);
                
                $cdretorno = 1;
                $dsobs = 'Desbloqueio realizado';
            }else{
                $cdretorno = 0;
                $dsobs = 'Nota não encontrada';
            }

            $notaDesbloqueada = array(
                "CDFILIAL" => $row['CDFILIAL'],
                "NRLANCTONF" => $row['NRLANCTONF'],
                "CDRETORNO" => $cdretorno,
                "DSOBS" => $dsobs,
            );
            
            return $notaDesbloqueada;
            
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function getNotaBloqueada($row){
        try{
            $queryNotas = "SELECT NRLANCTONF
                            FROM NOTAFISCAL
                            WHERE CDFILIAL = :P_CDFILIAL
                            AND NRLANCTONF = :P_NRLANCTONF";

			$params = array(
                'P_CDFILIAL'    => $row['CDFILIAL'],
                'P_NRLANCTONF'  => $row['NRLANCTONF']
            );
            
            $notas = $this->entityManager->getConnection()->fetchAll($queryNotas, $params);
            return $notas;
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function desbloqueiaNota($row){
        try{
            $desbloqueiaNota = "UPDATE NOTAFISCAL
                            SET IDBLOQUEADO = 'N'
                            WHERE CDFILIAL = :P_CDFILIAL
                            AND NRLANCTONF = :P_NRLANCTONF";

			$params = array(
                'P_CDFILIAL'    => $row['CDFILIAL'],
                'P_NRLANCTONF'  => $row['NRLANCTONF']
            );
            
            $resultQuery = $this->entityManager->getConnection()->executeQuery($desbloqueiaNota, $params);
        } catch (\Exception $e) {
            throw $e;
        }
    }
}